function [FOPT, CountEval, xopt, fopt, NIteration, DIV]=EPSO(Particle_Number,dim,Xmin,Xmax, nexp, fhd, sistema, nhc, pel,r, nhillC, Max_FES)

%% Detailed Inputs


% Particle_Number -- number of particles in swarm
% dim -------------- number of candicate equipment
% Xmin,Xmax -------- lower and uper bound for expansion (vectors)
% nexp ------------- number of optimization repetitions
% fhd -------------- function evaluation particles
% sistema ---------- system under study
% nhc -------------- Number of particles accelerated trhough the Hill Climbing
% pel -------------- Percentage of Elite set at selection block
% r ---------------- replication parameter
% nhillC ----------- number of iterations on Hill Climbing (new solutions)
% Max_FES ---------- maximum number of evaluations

%% Detailed Outputs
% FOPT ------------- Cell (T columns) with the best value per iteration for each trial   
% CountEval -------- Vector with fevalcount per trial
% xopt ------------- Best solution per trial
% fopt ------------- Value of the best solution per trial
% Niteration ------- Number of iterations per trial
% DIV -------------- Cell (T columns) Diversity of the population per iteration for each trial

%% -------------------------------------------------------------------------------------------------------------
for T = 1:nexp
    best_value_particle = [];
    best_particle = [];
    
    %% Initialization of population
    
    for i=1:dim
        for j=1:Particle_Number%-nhillC
            populacao(i,j) = randi([Xmin(i) Xmax(i)],1,1);
        end
    end
    
    iter = 1;       % Iteration counter
    fevalcount = 0; % Counter of objective function evaluations
    STOP = 0;       % Stop flag
    
    %% Evaluation of part of the first population
    
    parfor i=1:Particle_Number%-nhillC
        [costo_lineas, penalizacion]=feval(fhd, populacao(:,i), sistema);
        fswarm(i)=costo_lineas+penalizacion;
        fevalcount=fevalcount+1;
    end
    
    %% Creating local optima solutions on Hill Clibing
%     parfor i=Particle_Number-nhillC+1:Particle_Number
%         ind = Xmin;
%         [populacao(:,i),fswarm(i),fcount(i)] = HillClibingStart(fhd, ind, sistema, Xmax);
%     end
%     
%     fevalcount = fevalcount+sum(fcount);
    
    %% Diversity of the new population
    [~,ia,~]=unique(populacao','rows','stable');
    diversity(iter)=length(ia)*100/Particle_Number;
    
    %% Store gbest and pbest
    
    pbest = populacao;
    Vpbest = fswarm;
    
    [a1,b1] = sort (fswarm);
    gbest = populacao(:,b1(1));
    Vgbest = fswarm(1);
    
    %% Weights of replicated population
    
    wi1 = rand(Particle_Number,r);
    wi2 = rand(Particle_Number,r);
    wi3 = rand(Particle_Number,r);
    wi4 = rand();
    
    Vel11 = rand(dim,r);
    
    [best_value_particle(iter,1), ind] = min(Vgbest);
    best_particle(:,iter) = gbest(:,ind);
    %% LOOP OF EVOLUTION OF THE SWARM
    
    while STOP == 0
        iter = iter + 1;
        % REPLICATION BLOCK
        for rep=1:r
            POP{rep}=populacao;
        end
        
        %         % MUTATION BLOCK
        wi1 = (0.5+rand-(1./(1+exp(wi1))));
        wi2 = (0.5+rand-(1./(1+exp(wi2))));
        wi3 = (0.5+rand-(1./(1+exp(wi3))));
        wi4 = (0.5+rand-(1./(1+exp(wi4))));
        
        chi = 0.729; % Constriction coefficient
        c1 = 2.05;   % Cognitive parameter
        c2 = 2.05;   % Social parameter
        
        % UPDATE VELOCITY AND POSITION
        for rep=1:r
            for i=1:Particle_Number
                % Update velocity
                if rep == 1
                    R1 = rand(dim,1);
                    R2 = rand(dim,1);
                    Vel11(:,rep)=round(chi .*(Vel11(:,rep) + c1.*R1.*(pbest(:,i)-POP{1,rep}(:,i)) + c2.*R2.*(gbest-POP{1,rep}(:,i))));
                else
                    Vel11(:,rep)=round(Vel11(:,rep).*wi1(i,rep)*rand+wi2(i,rep).*(pbest(:,i)*rand-POP{1,rep}(:,i))+wi3(i,rep).*(gbest-POP{1,rep}(:,i))*rand);
                end
                
                if 1 && all(Vel11(:,rep) == 0)
                    Vel11(:,rep) = round(Xmax(1)*randn(dim,1)*(0.5+rand-1)/(2));
                end
                
                POP{1,rep}(:,i) = POP{1,rep}(:,i)+Vel11(:,rep);
                
                % Lower bound
                aux_lb = POP{1,rep}(:,i);
                aux_lb(aux_lb<Xmin)=Xmin(aux_lb<Xmin)+round(rand(size(Xmin(aux_lb<Xmin),1),1));
                
                POP{1,rep}(:,i) = aux_lb;
                % Upper bound
                aux_ub = POP{1,rep}(:,i);
                aux_ub(aux_ub>Xmax)=Xmax(aux_ub>Xmax)-round(rand(size(Xmax(aux_ub>Xmax),1),1));
                POP{1,rep}(:,i) = aux_ub;
            end
        end
        
        % EVALUATION BLOCK
        
        
        for rep=1:r
            populacao_new = POP{1,rep};
            parfor i=1:Particle_Number
                [costo_lineas(i), penalizacion(i)]=feval(fhd, populacao_new(:,i), sistema);
                fswarmPOP(rep,i)=costo_lineas(i)+penalizacion(i);
                fevalcount=fevalcount+1;
            end
        end
        
        % SELECTION BLOCK -
        sh = [fswarmPOP;fswarm];
        fav = [fswarmPOP;fswarm];
        [~,fswarmInd] = sort (fav);
        POP{1,r+1}=populacao;
        for i=1:Particle_Number
            vbests(i) = fav(fswarmInd(1,i),i);
        end
        [~,av2]=sort(vbests);
        Elite = av2(1:pel*Particle_Number);
        
        for i=1:Particle_Number
            if ismember(i,Elite)
                populacao_new(:,i)=POP{1,fswarmInd(1,i)}(:,i);
                fswarm_new(i) = fav(fswarmInd(1,i),i);
            else
                j = randi([1 rep+1],1,1);
                populacao_new(:,i)=POP{1,fswarmInd(j,i)}(:,i);
                fswarm_new(i) = fav(fswarmInd(j,i),i);
            end
        end
        
        %% Store gbest and pbest
        
        for i=1:Particle_Number
            if fswarm_new(i)<Vpbest(i)
                pbest(:,i) =  populacao_new(:,i);
                Vpbest(i) = fswarm_new(i);
            end
        end
        
        [a1,b1] = sort (fswarm_new);
        if fswarm_new(b1(1))<Vgbest
            gbest = populacao_new(:,b1(1));
            Vgbest = fswarm_new(b1(1));
        end
        
        %% Random particles accelerated on Hill Clibing
        VecRP = randi([1 Particle_Number],1,nhc);
        particle = populacao_new(:,VecRP);
        fit = fswarm_new(VecRP);
        parfor i=1:nhc
            [particle(:,i),fit(i),countg(i)] = HillClibingGbest(particle(:,i),fit(i),Xmin,sistema,fhd);
        end
        
        fevalcount=fevalcount+sum(countg);
        
        for i=1:nhc
            populacao_new(:,VecRP(i)) = particle(:,i);
            fswarm_new(VecRP(i)) = fit(i);
        end
        
        
        
        %% Diversity of the new population
        [~,ia,~]=unique(populacao_new','rows','stable');
        diversity(iter)=length(ia)*100/Particle_Number;
        
        %% NEW POPULATION AND STOP CRITERION
        populacao = populacao_new;
        fswarm = fswarm_new;
        
        [best_value_particle(iter,1), ind] = min(Vgbest);
        best_particle(:,iter) = gbest(:,ind);
        
        
        if fevalcount > Max_FES-(Particle_Number*r+dim*nhc) % (Particle_Number*r+dim*nhc) is the maximum number of evaluations per iteration
            STOP = 1;
        end
        
        
    end
    
    
    DIV{T}=diversity';
    FOPT{T}=best_value_particle;
    CountEval(T) = fevalcount;
    xopt(:,T) = best_particle(:,iter);
    fopt(T) = best_value_particle(iter);
    NIteration(T)=iter;
end


