%%% Script for initial parameters needed to solve the transmission
%%% expansion planning (TEP) problem by using AC or DC power flow calculation
%%% and also for running TEP (master script0


% Any publication resulting from the use of this m-file shall acknowledge
% it by citing the following paper:

%S. P. Torres and C. A. Castro, Expansion planning for smart transmission
%grids using ac model and shunt compensation, IET Generation,
%Transmission & Distribution, vol. 8, no. 5, pp. 966-975, 2014.
clear
close all
clc

for caso_estudo=1:4
    
    %% Test Systems and EPSO Parameters
    switch caso_estudo
        case 1
            sistema='garver';
            Particle_Number=20;                                            % Number of individuals
            Max_FES=5000;                                                  % Maximum number of objective function evaluations (alternative criterion)
            Xmax=5;                                                        % Maximum number of transmission lines that can be added in each right-of-way
            nhillC = Particle_Number/4;                                    % Particles initiated in local optimal locations - Hill Clibing
            nhc = Particle_Number/4;                                       % Number of particles accelerated trough the Hill Climbing at each iteration
            r = 3;                                                         % EPSO replication parameter
            pel = 0.6;                                                     % Percentage of Elite in the selection block
        case 2
            sistema='garver_WOrdptch';
            Particle_Number=20;                                            % Number of individuals
            Max_FES=5000;                                                  % Maximum number of objective function evaluations (alternative criterion)
            Xmax=5;                                                        % Maximum number of transmission lines that can be added in each right-of-way
            nhillC = Particle_Number/4;                                    % Particles initiated in local optimal locations - Hill Clibing
            nhc = Particle_Number/4;                                       % Number of particles accelerated trough the Hill Climbing at each iteration
            r = 3;                                                         % EPSO replication parameter
            pel = 0.6;                                                     % Percentage of Elite in the selection block
        case 3
            sistema='case24IEEE_AC';
            Particle_Number=40;                                            % Number of individuals
            Max_FES=12000;                                                 % Maximum number of objective function evaluations (alternative criterion)
            Xmax=5;                                                        % Maximum number of transmission lines that can be added in each right-of-way
            nhillC = Particle_Number/4;                                    % Particles initiated in local optimal locations - Hill Clibing
            nhc = Particle_Number/4;                                       % Number of particles accelerated trough the Hill Climbing at each iteration
            r = 3;                                                         % EPSO replication parameter
            pel = 0.6;                                                     % Percentage of Elite in the selection block
        case 4
            sistema='case118IEEE_AC20Renew';
            Particle_Number=100;                                           % Number of individuals
            Max_FES=50000;                                                 % Maximum number of objective function evaluations (alternative criterion)
            Xmax=8;                                                        % Maximum number of transmission lines that can be added in each right-of-way
            nhillC = Particle_Number/4;                                    % Particles initiated in local optimal locations - Hill Clibing
            nhc = 10;                                                      % Number of particles accelerated trough the Hill Climbing at each iteration
            r = 3;                                                         % EPSO replication parameter
            pel = 0.6;                                                     % Percentage of Elite in the selection block
    end
    
    %% Other parameters
    nexp=31;                                  % Number of optimization repetitions
    mpc=loadcase(sistema);                   % Load system data (defined in separate m-file)
    LineasIniciales=mpc.costo_lineas(:,1);
    Costos=mpc.costo_lineas(:,2);            % Store line cost data
    dim=numel(LineasIniciales);              %Problem dimension (defined by number of right-of-way considered for expansion)
    Xmax = repmat(Xmax, dim, 1);
    Xmin = LineasIniciales;
    fhd=@evaluacion_particulas;
    
    
    tic % Start measuring cpu time
    
    %% Call optimization function(defined by user/algorithm developer)
    
    [FOPT, CountEval, xopt, fopt, NIteration, DIV]=EPSO(Particle_Number,dim,Xmin,Xmax, nexp, fhd, sistema, nhc, pel,r, nhillC, Max_FES);
    
    t(caso_estudo)=toc; % Stop measuring cpu time
    name = num2str(caso_estudo);
    save(name, 'FOPT', 'CountEval', 'xopt', 'fopt', 'NIteration','t','DIV') % Store results of selected vairables in .mat archive
    
end